local _, namespace = ...
local categories = namespace.options.categories
local SetActiveFrame
local icons = {}
local ARENA_GUIDS = {}

if not IsAddOnLoaded("Blizzard_ArenaUI") then
	LoadAddOn("Blizzard_ArenaUI") -- Originally meant to be loaded through .TOC
end

do
	local GetTime, GetSpellTexture = GetTime, GetSpellTexture

	local function SetIndicators(frame, testMode)
		local applied = not testMode and frame.applied + 1 or random(1, 3)
		frame.applied = applied

		if frame.border then
			frame.border:SetVertexColor(1, applied == 2 and 1 or 0, 0, applied > 1 and 1 or 0)
		end

		if frame.text then
			frame.text:SetTextColor(1, applied <= 2 and 1 or 0, applied == 1 and 1 or 0, 1)
		end
	end

	function SetActiveFrame(frame, spellID, testMode)
		if not frame then return end -- sanity check
		if not frame.hasTexture then
			frame.icon:SetTexture(GetSpellTexture(spellID))
		end

		SetIndicators(frame, testMode)
		frame.time = GetTime()
		frame:SetCooldown(frame.time, not testMode and 18 or random(6, 18))
	end
end

do
	local UnitGUID, GetArenaOpponentSpec = UnitGUID, GetArenaOpponentSpec

	local function GetOpponentNumber(self, destGUID)
		if not destGUID then return end
		local oppNumber = 0

		for i = 1, 5 do
			if UnitGUID("arena"..i) == destGUID then
				oppNumber = i
				break
			elseif UnitGUID("arenapet"..i) == destGUID then
				if GetArenaOpponentSpec(i) == 253 then -- BM hunter shares DR with pet
					oppNumber = i
					break
				end
			end
		end

		self[destGUID] = oppNumber -- note: 0 will be treated as a blacklist
		return oppNumber
	end

	-- "memoization" for guids
	setmetatable(ARENA_GUIDS, {__index = GetOpponentNumber})
end

do
	local _, englishClass = UnitClass("player")
	local options = namespace.options
	local iconSize = options.iconSize
	local reverse = options.iconReverse
	local DEFAULT_POSITION_X = not reverse and -24 or 100

	local function FrameOnShow(self)
		local posX = DEFAULT_POSITION_X
		local oppNumber = self.oppNumber

		for i = 1, #categories do
			if icons[categories[i]..oppNumber].shown then
				posX = not reverse and posX - iconSize or posX + iconSize
			end
		end

		self:SetPoint("LEFT", posX, 22)
		self.shown = true
	end

	local function FrameOnHide(self)
		local posX = DEFAULT_POSITION_X
		local oppNumber = self.oppNumber
		local time = self.time
		self.shown = false
		self.applied = 0

		for i = 1, #categories do
			local frame = icons[categories[i]..oppNumber]

			if frame.shown then
				if frame.time > time or frame:GetCooldownDuration() > self:GetCooldownDuration() then
					frame:SetPoint("LEFT", posX, 22)
					posX = not reverse and posX - iconSize or posX + iconSize
				end
			end
		end

		if self:GetCooldownDuration() > 0 then
			self:SetCooldown(0, 0) -- Stop timers when hidden by parent frame (leaving arena etc)
		end
	end

	for i = 1, 5 do
		local arenaFrame = _G["ArenaEnemyFrame"..i]

		for j = 1, #categories do
			local frame = CreateFrame("Cooldown", nil, arenaFrame, "CooldownFrameTemplate")
			frame.time, frame.applied, frame.oppNumber, frame.shown = 0, 0, i, false
			frame:ClearAllPoints()
			frame:SetHideCountdownNumbers(options.timerTextOff)
			frame:SetSize(options.iconSize, options.iconSize)
			frame:SetDrawSwipe(options.timerSpiral)
			frame:SetDrawEdge(false)
			frame:EnableMouse(false)
			frame:SetFrameLevel(2)
			frame:SetSwipeColor(0, 0, 0, 0.65)
			frame:Hide()

			frame.icon = frame:CreateTexture(nil, "BACKGROUND")
			frame.icon:SetAllPoints(frame)
			frame.icon:SetTexture(options.textures[englishClass] and options.textures[englishClass][categories[j]])
			frame.hasTexture = frame.icon:GetTexture() and true or false

			if options.borderColors then
				frame.border = frame:CreateTexture(nil, "OVERLAY")
				frame.border:SetPoint("TOPLEFT", -1, 1)
				frame.border:SetPoint("BOTTOMRIGHT", 1, -1)
				frame.border:SetTexture("Interface\\BUTTONS\\UI-Quickslot-Depress")
			end

			for _, region in next, {frame:GetRegions()} do
				if region:GetObjectType() == "FontString" then
					region:SetFont("Fonts\\FRIZQT__.TTF", options.timerTextSize, "OUTLINE")
					region:SetPoint("CENTER", frame, 0, 0)
					frame.text = not options.timerTextOff and options.timerColors and region or false
					break
				end
			end

			frame:SetScript("OnShow", FrameOnShow)
			frame:SetScript("OnHide", FrameOnHide)
			icons[categories[j]..i] = frame
		end
	end

	namespace.options = nil
end

do
	local spellIDs = namespace.spellList
	local events = CreateFrame("Frame")

	local function EventHandler(self, event, _, eventType, _, _, _, srcFlags, _, destGUID, _, _, _, spellID, _, _, auraType) -- avoid vararg for unnecessary table
		if event == "COMBAT_LOG_EVENT_UNFILTERED" then
			if (eventType == "SPELL_AURA_REMOVED" or eventType == "SPELL_AURA_REFRESH") and auraType == "DEBUFF" then
				local category = spellIDs[spellID]
				if category then
					local oppNumber = ARENA_GUIDS[destGUID]
					if oppNumber and oppNumber > 0 then
						SetActiveFrame(icons[category..oppNumber], spellID)
						return
					end
				end
			end
		else
			if select(2, IsInInstance()) == "arena" then
				wipe(ARENA_GUIDS)
				self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
			else
				self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
			end
		end
	end

	events:RegisterEvent("PLAYER_ENTERING_WORLD")
	events:SetScript("OnEvent", EventHandler)
	events:Hide()
end

SLASH_DIMINISH1 = "/diminish"
SlashCmdList["DIMINISH"] = function()
	if InCombatLockdown() or UnitAffectingCombat("player") then
		print("Must leave combat")
	return end

	local showFlag = not ArenaEnemyFrames:IsShown()
	ArenaEnemyFrames:SetShown(showFlag)
	for i = 1, 5 do
		_G["ArenaEnemyFrame"..i]:SetShown(showFlag)
		if showFlag then
			for j = 1, #categories do
				SetActiveFrame(icons[categories[j]..i], 118, true)
			end
		end
	end
end